/*
 * $Id: rb_evas.c 43 2004-07-26 10:49:51Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#define __RB_EVAS_C
#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

VALUE cEvas;
static VALUE parents;

static void c_mark (Evas **e)
{
	VALUE parent;

	parent = rb_hash_aref (parents, INT2NUM ((long) (e)));
	if (parent != Qnil)
		rb_gc_mark (parent);
}

static void c_free (Evas **e)
{
	/* do NOT call evas_free() here, since the Evas is freed
	 * by ecore_evas_free() eventually
	 */
	rb_hash_aset (parents, INT2NUM ((long) e), Qnil);

	free (e);
}

VALUE TO_EVAS (VALUE parent, Evas *e)
{
	VALUE self;
	Evas **my_e = NULL;

	if (NIL_P (parent) || !e)
		return Qnil;

	self = Data_Make_Struct (cEvas, Evas *,
	                         c_mark, c_free, my_e);
	*my_e = e;

	rb_hash_aset (parents, INT2NUM ((long) my_e), parent);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_inspect (VALUE self)
{
	INSPECT (self, Evas *);
}

static VALUE c_render (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	evas_render (*e);

	return Qnil;
}

static VALUE c_font_path_clear (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	evas_font_path_clear (*e);

	return Qnil;
}

static VALUE c_font_path_append (VALUE self, VALUE path)
{
	GET_OBJ (self, Evas *, e);

	Check_Type (path, T_STRING);

	evas_font_path_append (*e, StringValuePtr (path));

	return Qnil;
}

static VALUE c_font_path_prepend (VALUE self, VALUE path)
{
	GET_OBJ (self, Evas *, e);

	Check_Type (path, T_STRING);

	evas_font_path_append (*e, StringValuePtr (path));

	return Qnil;
}

static VALUE c_font_path_get (VALUE self)
{
	VALUE ary;
	const Evas_List *list, *l;

	GET_OBJ (self, Evas *, e);

	if (!(list = evas_font_path_list (*e)))
		return rb_ary_new ();

	ary = rb_ary_new2 (evas_list_count ((Evas_List *) list));

	for (l = list; l; l = l->next)
		rb_ary_push (ary, rb_str_new2 (l->data));

	return ary;
}

static VALUE c_font_cache_get (VALUE self)
 {
	GET_OBJ (self, Evas *, e);

	return INT2FIX (evas_font_cache_get (*e));
}

static VALUE c_font_cache_set (VALUE self, VALUE val)
{
	GET_OBJ (self, Evas *, e);

	Check_Type (val, T_FIXNUM);

	evas_font_cache_set (*e, FIX2INT (val));

	return Qnil;
}

static VALUE c_font_cache_flush (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	evas_font_cache_flush (*e);

	return Qnil;
}

static VALUE c_image_cache_get (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	return INT2FIX (evas_image_cache_get (*e));
}

static VALUE c_image_cache_set (VALUE self, VALUE val)
{
	GET_OBJ (self, Evas *, e);

	Check_Type (val, T_FIXNUM);

	evas_image_cache_set (*e, FIX2INT (val));

	return Qnil;
}

static VALUE c_image_cache_reload (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	evas_image_cache_reload (*e);

	return Qnil;
}

static VALUE c_image_cache_flush (VALUE self)
{
	GET_OBJ (self, Evas *, e);

	evas_image_cache_flush (*e);

	return Qnil;
}

static VALUE c_top_get (VALUE self)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas *, e);

	if (!(o = evas_object_top_get (*e)))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

static VALUE c_bottom_get (VALUE self)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas *, e);

	if (!(o = evas_object_bottom_get (*e)))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

static VALUE c_find_object (VALUE self, VALUE name)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas *, e);

	Check_Type (name, T_STRING);

	if (!(o = evas_object_name_find (*e, StringValuePtr (name))))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

void Init_Evas (void)
{
	cEvas = rb_define_class_under (mEvas, "Evas", rb_cObject);

	/* not publically instantiable yet */
	rb_define_private_method (rb_singleton_class (cEvas),
	                          "new", NULL, 0);
	rb_define_method (cEvas, "inspect", c_inspect, 0);
	rb_define_method (cEvas, "render", c_render, 0);
	rb_define_method (cEvas, "font_path_clear", c_font_path_clear, 0);
	rb_define_method (cEvas, "font_path_append", c_font_path_append, 1);
	rb_define_method (cEvas, "font_path_prepend", c_font_path_prepend, 1);
	rb_define_method (cEvas, "font_path", c_font_path_get, 0);
	rb_define_method (cEvas, "font_cache", c_font_cache_get, 0);
	rb_define_method (cEvas, "font_cache=", c_font_cache_set, 1);
	rb_define_method (cEvas, "font_cache_flush",
	                  c_font_cache_flush, 0);
	rb_define_method (cEvas, "image_cache", c_image_cache_get, 0);
	rb_define_method (cEvas, "image_cache=", c_image_cache_set, 1);
	rb_define_method (cEvas, "image_cache_reload",
	                  c_image_cache_reload, 0);
	rb_define_method (cEvas, "image_cache_flush",
	                  c_image_cache_flush, 0);
	rb_define_method (cEvas, "top", c_top_get, 0);
	rb_define_method (cEvas, "bottom", c_bottom_get, 0);
	rb_define_method (cEvas, "find_object", c_find_object, 1);

	parents = rb_hash_new ();
	rb_global_variable (&parents);
}
