/*
 * $Id: rb_evas_object.c 46 2004-07-26 11:14:50Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#define __RB_EVAS_OBJECT_C
#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

VALUE cEvasObject;
static VALUE parents;

/* called by the child classes */
void c_evas_object_free (Evas_Object **e)
{
	if (*e)
		evas_object_del (*e);

	rb_hash_aset (parents, INT2NUM ((long) e), Qnil);

	free (e);
}

void c_evas_object_mark (Evas_Object **e)
{
	VALUE parent;

	parent = rb_hash_aref (parents, INT2NUM ((long) (e)));
	if (!NIL_P (parent))
		rb_gc_mark (parent);
}

static VALUE c_init (VALUE self, VALUE parent)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_data_set (*e, RUBY_EVAS_OBJECT_KEY, (void *) self);

	rb_hash_aset (parents, INT2NUM ((long) e), parent);

	return self;
}

static VALUE c_inspect (VALUE self)
{
	INSPECT (self, Evas_Object *);
}

static VALUE c_delete (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_del (*e);
	*e = NULL;

	return Qnil;
}

static VALUE c_resize (VALUE self, VALUE w, VALUE h)
{
	GET_OBJ (self, Evas_Object *, e);

	Check_Type (w, T_FIXNUM);
	Check_Type (h, T_FIXNUM);

	evas_object_resize (*e, (Evas_Coord) FIX2INT (w),
	                        (Evas_Coord) FIX2INT (h));

	return Qnil;
}

static VALUE c_move (VALUE self, VALUE x, VALUE y)
{
	GET_OBJ (self, Evas_Object *, e);

	Check_Type (x, T_FIXNUM);
	Check_Type (y, T_FIXNUM);

	evas_object_move (*e, (Evas_Coord) FIX2INT (x),
	                      (Evas_Coord) FIX2INT (y));

	return Qnil;
}

static VALUE c_geometry_get (VALUE self)
{
	int x = 0, y = 0, w = 0, h = 0;

	GET_OBJ (self, Evas_Object *, e);

	evas_object_geometry_get (*e, (Evas_Coord *) &x, (Evas_Coord *) &y,
	                          (Evas_Coord *) & w, (Evas_Coord *) &h);

	return rb_ary_new3 (4, INT2FIX (x), INT2FIX (y), INT2FIX (w),
	                    INT2FIX (h));
}

static VALUE c_show (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_show (*e);

	return Qnil;
}

static VALUE c_hide (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_hide (*e);

	return Qnil;
}

static VALUE c_visible_get (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	return evas_object_visible_get (*e) ? Qtrue : Qfalse;
}

static VALUE c_evas_get (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	return rb_hash_aref (parents, INT2NUM ((long) (e)));
}

static VALUE c_name_get (VALUE self)
{
	const char *name;

	GET_OBJ (self, Evas_Object *, e);

	if (!(name = evas_object_name_get (*e)))
		return Qnil;
	else
		return rb_str_new2 (name);
}

static VALUE c_name_set (VALUE self, VALUE val)
{
	GET_OBJ (self, Evas_Object *, e);

	Check_Type (val, T_STRING);

	evas_object_name_set (*e, StringValuePtr (val));

	return Qnil;
}

static VALUE c_layer_get (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	return INT2FIX (evas_object_layer_get (*e));
}

static VALUE c_layer_set (VALUE self, VALUE val)
{
	GET_OBJ (self, Evas_Object *, e);

	Check_Type (val, T_FIXNUM);

	evas_object_layer_set (*e, NUM2INT (val));

	return Qnil;
}

static VALUE c_get_color (VALUE self)
{
	int r = 0, g = 0, b = 0, a = 0;

	GET_OBJ (self, Evas_Object *, e);

	evas_object_color_get (*e, &r, &g, &b, &a);

	return rb_ary_new3 (4, INT2FIX (r), INT2FIX (g), INT2FIX (b),
	                    INT2FIX (a));
}

static VALUE c_set_color (VALUE self, VALUE r, VALUE g, VALUE b,
                          VALUE a)
{
	GET_OBJ (self, Evas_Object *, e);

	Check_Type (r, T_FIXNUM);
	Check_Type (g, T_FIXNUM);
	Check_Type (b, T_FIXNUM);
	Check_Type (a, T_FIXNUM);

	evas_object_color_set (*e, FIX2INT (r), FIX2INT (g), FIX2INT (b),
	                       FIX2INT (a));

	return Qnil;
}

static VALUE c_pass_events_get (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	return evas_object_pass_events_get (*e) ? Qtrue : Qfalse;
}

static VALUE c_pass_events_set (VALUE self, VALUE val)
{
	GET_OBJ (self, Evas_Object *, e);

	CHECK_BOOL (val);

	evas_object_pass_events_set (*e, val == Qtrue ? 1 : 0);

	return Qnil;
}

static VALUE c_raise (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_raise (*e);

	return Qnil;
}

static VALUE c_lower (VALUE self)
{
	GET_OBJ (self, Evas_Object *, e);

	evas_object_lower (*e);

	return Qnil;
}

static VALUE c_stack_above (VALUE self, VALUE target)
{
	GET_OBJ (self, Evas_Object *, e);

	if (!rb_obj_is_kind_of (target, cEvasObject)) {
		rb_raise (rb_eTypeError,
		          "wrong argument type %s (expected EvasObject)",
		          rb_obj_classname (target));
		return Qnil;
	}

	GET_OBJ (target, Evas_Object *, target2);

	evas_object_stack_above (*e, *target2);

	return Qnil;
}

static VALUE c_stack_below (VALUE self, VALUE target)
{
	GET_OBJ (self, Evas_Object *, e);

	if (!rb_obj_is_kind_of (target, cEvasObject)) {
		rb_raise (rb_eTypeError,
		          "wrong argument type %s (expected EvasObject)",
		          rb_obj_classname (target));
		return Qnil;
	}

	GET_OBJ (target, Evas_Object *, target2);

	evas_object_stack_below (*e, *target2);

	return Qnil;
}

static VALUE c_above_get (VALUE self)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas_Object *, e);

	if (!(evas_object_above_get (*e)))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

static VALUE c_below_get (VALUE self)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas_Object *, e);

	if (!(evas_object_below_get (*e)))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

void Init_EvasObject (void)
{
	cEvasObject = rb_define_class_under (mEvas, "EvasObject",
	                                     rb_cObject);

	rb_define_private_method (rb_singleton_class (cEvasObject),
	                          "new", NULL, 0);
	rb_define_method (cEvasObject, "initialize", c_init, 1);
	rb_define_method (cEvasObject, "inspect", c_inspect, 0);
	rb_define_method (cEvasObject, "delete", c_delete, 0);
	rb_define_method (cEvasObject, "resize", c_resize, 2);
	rb_define_method (cEvasObject, "move", c_move, 2);
	rb_define_method (cEvasObject, "geometry", c_geometry_get, 0);
	rb_define_method (cEvasObject, "show", c_show, 0);
	rb_define_method (cEvasObject, "hide", c_hide, 0);
	rb_define_method (cEvasObject, "visible?", c_visible_get, 0);
	rb_define_method (cEvasObject, "evas", c_evas_get, 0);
	rb_define_method (cEvasObject, "name", c_name_get, 0);
	rb_define_method (cEvasObject, "name=", c_name_set, 1);
	rb_define_method (cEvasObject, "layer", c_layer_get, 0);
	rb_define_method (cEvasObject, "layer=", c_layer_set, 1);
	rb_define_method (cEvasObject, "get_color", c_get_color, 0);
	rb_define_method (cEvasObject, "set_color", c_set_color, 4);
	rb_define_method (cEvasObject, "pass_events?", c_pass_events_get, 0);
	rb_define_method (cEvasObject, "pass_events=", c_pass_events_set, 1);
	rb_define_method (cEvasObject, "raise", c_raise, 0);
	rb_define_method (cEvasObject, "lower", c_lower, 0);
	rb_define_method (cEvasObject, "stack_above", c_stack_above, 1);
	rb_define_method (cEvasObject, "stack_below", c_stack_below, 1);
	rb_define_method (cEvasObject, "above", c_above_get, 0);
	rb_define_method (cEvasObject, "below", c_below_get, 0);

	parents = rb_hash_new ();
	rb_global_variable (&parents);
}
